/*******************************************************************************
*                     Copyright (c) Gemplus Card International 1996
*
* Name        : MPCOS Smart Card Utilities (WSCUMP2.H)
*
* Description : Smart Card Utilities for MPCOS cards.
*
  Compiler    : Microsoft Visual C++ 1.5x

  Host        : IBM PC and compatible machines under Windows 3.x.

* Release     : 3.1
*
* Date        : 08/30/93 Start of release 1.0
* Last Modif  : 21/05/96, release 3.1
*
*******************************************************************************/

#ifndef _MPCOS_H_
#define _MPCOS_H_

/*----------------------------------------------------------------------------*/
/* define section                                                             */
/*----------------------------------------------------------------------------*/

#ifndef FALSE
   #define FALSE              0
   #define TRUE               1
#endif

#ifndef PLAIN
   #define DECIPHER           2
   #define CIPHER             1
   #define PLAIN              0
#endif

#define EF                    2  /* Used by G_MPC_Create...                */
#define DF                    1  /* Used by G_MPC_Create...                */

#define MF                    0
#define LOCAL                 1

#define ADM                   0  /* Used by G_MPC_LoadKey...               */
#define PAY                   1  /* Used by G_MPC_LoadKey...               */
#define MAC                                       2  /* Used by G_MPC_LoadKey...               */  

#define NO_BATCH_KEY          0  /* Type of Batch Key           */
#define BATCH_CIPHER          1  
#define BATCH_DECIPHER        2  
#define BATCH_MAC             3  

#define ISO_DF                0  /* Used by G_MPC_Create...                */
#define PCOS_DF               1  /* Used by G_MPC_Create...                */
#define SUBDBT                0  /* Used by G_MPC_Create...                */
#define NO_SUBDBT             2  /* Used by G_MPC_Create...                */
#define CUR_BAL_SIGN          4  /* Used by G_MPC_Create...                */
#define NO_CUR_BAL_SIGN       0  /* Used by G_MPC_Create...                */
#define Unblock_ON            8  /* Used by G_MPC_Create...                */
#define UpdBin_ON          0x10  /* Used by G_MPC_Create...                */
#define RdBin_ON           0x20  /* Used by G_MPC_Create...                */
#define CreateFreeze_ON    0x40  /* Used by G_MPC_Create...                */
#define Credit_ON          0x80  /* Used by G_MPC_Create...                */

#define ADM_SESSION           1
#define PAY_SESSION           2

#define NO_SECURE             0
#define SECURE                1


#define NO_LOCKED             0  /* Used by G_MPC_Freeze...                */
#define LOCKED                1  /* Used by G_MPC_Freeze...                */
#define NO_CHANGE             0  /* Used by G_MPC_Freeze...                */

#define UNBLOCK               1  /* Used by G_MPC_SetCode...               */
#define CHANGE                0  /* Used by G_MPC_SetCode...               */

#define SELECT                1  /* Used by G_MPC_ReadBin...               */
#define NO_SELECT             0  /* Used by G_MPC_ReadBin...               */

#define EF_TRANSPARENT     0x01  /* Used by G_MPC_CreateEF...              */
#define EF_MANAGER         0x11  /* Used by G_MPC_CreateEF...              */
#define EF_PURSE           0x19  /* Used by G_MPC_CreateEF...              */
#define EF_CODE            0x21  /* Used by G_MPC_CreateEF...              */
#define EF_KEY             0x29  /* Used by G_MPC_CreateEF...              */

#define T_0                   0  /* Used by G_MPC_SwitchProtocol...        */
#define T_1                   2  /* Used by G_MPC_SwitchProtocol...        */
#define T_14                  3  /* Used by G_MPC_SwitchProtocol...        */

#define AUTH                 16
#define CALC                  8
#define NO_CARD_RND           1
#define CARD_RND              0          
#define NO_RND                0
#define ONE_RND               8
#define TWO_RND              16

#define FULL_TRIPLE_DES       3  /* Cryptographic options                  */
#define INVA_OPTION_DES       1  /*                                        */

#define UNKNOWN_CHIP     0xFFFF  /* Chip type                              */
#define MPCOS8K                1  
#define MPCOS24K               2  
#define MPCOS32K               4  
#define MPCOS64K               3  

/**** Constants ****/
#define  G_OK                                     0                                             
#define  UNKNOW_CODE         22222



/*----------------------------------------------------------------------------*/
/* Error Constants                                                            */
/*----------------------------------------------------------------------------*/
#define GE_FMT_LENGTH         -2020   /* 6700 : On the Length (P3)         */
#define GE_FMT_REF            -2030   /* 6B00 : On P1 and/or P2            */
#define GE_FMT_INS            -2040   /* 6D00 : Instruction not supported  */
#define GE_FMT_CLA            -2050   /* 6E00 : On Class of application    */

#define GE_MEM_PROG           -2110   /* 6581 * 9202 : Can't Write/Update the mem */
#define GE_MEM_OUT_OF_RANGE   -2120   /* 6A84 : Insufisant space for create       */


#define GE_SEC_NOACCESS_BIS   -2232   /* 9806 : Debit Access condition not     */
				      /* fullfilled in SUBSTITUTION step       */
#define GE_FIL_NOT_FOUND      -2310   /* 6A82 : The entity is not found        */
#define GE_FIL_INTEGRITY      -2320   /* 6284 * 9400 : Parity / Checksum wrong */
#define GE_FIL_CONTEXT        -2370   /* 6985 : File not exist or not select   */

#define GE_BALANCE_ERROR      -2410   /* 9100 : The balance is Wrong       */
#define GE_BALANCE_ERROR_BIS  -2412   /* 9102 : Insufficient Balance in    */
				      /* SUBSTITUTION step                 */
#define GE_PURSE_ERROR        -2420   /* 9404 : Purse selection error      */

#define GE_KEY_ERROR          -2430   /* 9408 : Key selection error        */
#define GE_DAT_WRONG          -2470   /* 6A80 : Data or checksum are wrong */

#define GE_SEQUENCE           -2500   /* 6900 : The sequence is wrong      */
#define GE_CMD_INCOMPATIBLE   -2610   /* 6981 : Incompatible with context  */

#define GE_SECURITY           -4000   /* 9810 : Issuer secret code not entered */
#define GE_SEC_LOCKED         -4010   /* 6983 : Code is locked             */
#define GE_SEC_NOACCESS       -4030   /* 6982 : Access Condition not verify*/
#define GE_SEC_NOT_SESSION    -4040   /* 9820 : No set session             */
#define GE_SEC_NOT_FOUND      -4050   /* 6984 : Code or key not found      */
#define GE_SEC_NOT_VALID      -4060   /* 9802 : Code can't be used         */

#define GE_SEC_FAILED         -4200   /* 6300 : Code, Key, Crypto is wrong */
#define GE_SEC_FAILED_ONE     -4201   /* 6301 : False verify code    */
#define GE_SEC_FAILED_TWO     -4202   /* 6303 : False verify code    */
#define GE_SEC_FAILED_THREE   -4203   /* 6307 : False verify code    */
#define GE_SEC_FAILED_FOUR    -4204   /* 630F : False verify code    */
#define GE_SEC_FAILED_FIVE    -4205   /* 631F : False verify code    */
#define GE_SEC_FAILED_SIX     -4206   /* 633F : False verify code    */
#define GE_SEC_FAILED_SEVEN   -4207   /* 637F : False verify code    */

#define GE_SEC_NONE_LEFT      -4500   /* 63C0  : False verify code   */
#define GE_SEC_ONE_LEFT       -4501   /* 63C1  : Left one verify     */
#define GE_SEC_TWO_LEFT       -4502   /* 63C2  : Left two verify     */
#define GE_SEC_THREE_LEFT     -4503   /* 63C3  : Left three verify   */
#define GE_SEC_FOUR_LEFT      -4504   /* 63C4  : Left four verify    */
#define GE_SEC_FIVE_LEFT      -4505   /* 63C5  : Left five verify    */
#define GE_SEC_SIX_LEFT       -4506   /* 63C6  : Left six verify     */


#define GE_EXP_LENGTH_032     -5032   /* 6C20 : Max Length accepted 32 bytes */
#define GE_EXP_LENGTH_035     -5035   /* 6C23 : Max Length accepted 32 bytes */
#define GE_EXP_LENGTH_096     -5096   /* 6C60 : Max Length accepted 96 bytes */
#define GE_EXP_LENGTH_099     -5099   /* 6C63 : Max Length accepted 99 bytes */

				       
/*----------------------------------------------------------------------------*/
/* structure definition section                                               */
/*----------------------------------------------------------------------------*/

typedef struct              /* ACCESS CONDITIONS structure                 */
{
   WORD16 Validity;         /*  Validity field of access condition         */
   WORD16 KeyLevel;         /*  Level of Key                               */
   WORD16 KeyShortID;       /*  Short Identifier of Key File               */
   WORD16 Sc1Level;         /*  Level of first secret code                 */
   WORD16 Sc1Number;        /*  Secret code number of first secret code    */
   WORD16 Sc2Level;         /*  Level of second secret code                */
   WORD16 Sc2Number;        /*  Secret code number of second secret code   */
} ACC;

typedef struct              /* ACCESS CONDITION to FREEZE                  */
{
   WORD16 Lock;             /*  Request to lock the EF                     */
   WORD16 KeyLevel;         /*  Level of Key                               */
   WORD16 Sc1Level;         /*  Level of first secret code                 */
   WORD16 Sc2Level;         /*  Level of second secret code                */
} ACF;

/*-------------------------------------------------------------------------*/
/* prototype section                                                       */
/*-------------------------------------------------------------------------*/


INT16  G_DECL G_MPC_InitLib         ( const WORD16, BYTE G_FAR *         );
INT16  G_DECL G_MPC_GetCardCode     (       WORD16 G_FAR *CardCode       );
void   G_DECL G_MPC_DisableCardCode ( void                               ); 
INT16  G_DECL G_MPC_SetChipType( const WORD16 type );


INT16  G_DECL G_MPC_GetResponse     ( const WORD16       Reader,
				      const WORD16       Length,
					    WORD8  G_FAR *Response
				    );

void G_DECL G_MPC_LoadACF           (       ACF    G_FAR *Grp,
					    INT16        Lock,
					    INT16        KeyLevel,
					    INT16        Sc1Level,
					    INT16        Sc2Level
				    ); 


INT16 G_DECL G_MPC_LoadCodePCOS (const WORD16 Reader,
				 const WORD16 CodeFile,
				 const INT16  CodeNumber,
				 const INT16  TypeCode,
				 const INT16  Ratif,
				 const INT16  UnblockLevel,
				 const INT16  UnblockCodeNumber,
				       BYTE   G_FAR *CodeValue
				);      
				
INT16 G_DECL G_MPC_LoadCode (const WORD16  Reader,
			     const INT16  Mode,
			     const WORD16 CodeFile,
			     const INT16  CodeNumber,
			     const INT16  TypeCode,
			     const INT16  Ratif,
			     const INT16  UnblockLevel,
			     const INT16  UnblockCodeNumber,
				   BYTE   G_FAR *CodeValue
			    );  
			    
INT16 G_DECL G_MPC_LoadCodeSecure (const WORD16  Reader,
				   const INT16  SMType,
				   const INT16  Mode,
				   const WORD16 CodeFile,
				   const INT16  CodeNumber,
				   const INT16  TypeCode,
				   const INT16  Ratif,
				   const INT16  UnblockLevel,
				   const INT16  UnblockCodeNumber,
					 BYTE   G_FAR *CodeValue,
					 BYTE   G_FAR *MACOut
				  );
				
INT16 G_DECL G_MPC_LoadCounterManager (const WORD16  Reader,
				       const INT16  CounterManagerFile,
				       const INT32  CardTransactionCounter,
				       const INT32  OnLineCounter
														      );
				      
INT16 G_DECL G_MPC_LoadCounterManagerSecure (const WORD16  Reader,
					     const INT16  SMType,
					     const INT16  CounterManagerFile,
					     const INT32  CardTransactionCounter,
					     const INT32  OnLineCounter,
						   BYTE  G_FAR *MACIn,
						   BYTE  G_FAR *MACOut
					    );
 
INT16 G_DECL G_MPC_LoadShortCM (const WORD16  Reader,
				const INT16   CounterManagerFile,
				const INT32   CardTransactionCounter
			       );

INT16 G_DECL G_MPC_LoadShortCMSecure (const WORD16  Reader,
				      const INT16  SMType,
				      const INT16  CounterManagerFile,
				      const INT32  CardTransactionCounter,
					    BYTE  G_FAR *MACIn,
					    BYTE  G_FAR *MACOut
				     );                               

INT16 G_DECL G_MPC_UpdateShortCM (const WORD16  Reader,
				  const INT16   CounterManagerFile,
				  const INT32   CardTransactionCounter
				 );
				 
INT16 G_DECL G_MPC_UpdateShortCMSecure (const WORD16  Reader,
					const INT16  SMType,
					const INT16  CounterManagerFile,
					const INT32  CardTransactionCounter,
					      BYTE  G_FAR *MACIn,
					      BYTE  G_FAR *MACOut
				       );  
				       
INT16 G_DECL G_MPC_UpdateCounterManager (const WORD16  Reader,
					 const INT16  CounterManagerFile,
					 const INT32  CardTransactionCounter,
					 const INT32  OnLineCounter
					);
					
INT16 G_DECL G_MPC_UpdateCounterManagerSecure (const WORD16  Reader,
					       const INT16  SMType,
					       const INT16  CounterManagerFile,
					       const INT32  CardTransactionCounter,
					       const INT32  OnLineCounter,
						     BYTE  G_FAR *MACIn,
						     BYTE  G_FAR *MACOut
					      );                                                                                                                                      
								  
INT16 G_DECL G_MPC_LoadPurseStandart( const WORD16       Reader,
				      const INT16        PurseFile,
				      const INT32        MaxAllowedBalance,
				      const INT16        CreditKeyFile,
				      const INT32        MaxFreeDebit,
				      const INT16        DebitCodeNumber,
				      const INT16        ReadCodeNumber
				    );
INT16 G_DECL G_MPC_LoadPurseStandartSecure(const WORD16  Reader,
				      const INT16        SMType,
				      const INT16        PurseFile,
				      const INT32        MaxAllowedBalance,
				      const INT16        CreditKeyFile,
				      const INT32        MaxFreeDebit,
				      const INT16        DebitCodeNumber,
				      const INT16        ReadCodeNumber,
					    BYTE   G_FAR *MACIn,
					    BYTE   G_FAR *MACOut
				    );
INT16 G_DECL G_MPC_LoadPurseExtensive (const WORD16      Reader,
				       const INT16       PurseFile,
				       const INT32       MaxAllowedBalance,
				       const INT16       CreditKeyFile,
				       const INT32       MaxFreeDebit,
				       const INT16       DebitCodeNumber,
				       const INT16       ReadCodeNumber,
				       const INT16       AccessCondLevel,
				       const INT16       CreditCodeNumber
				     );
INT16 G_DECL G_MPC_LoadPurseExtensiveSecure (const WORD16 Reader,
				       const INT16       SMType,
				       const INT16       PurseFile,
				       const INT32       MaxAllowedBalance,
				       const INT16       CreditKeyFile,
				       const INT32       MaxFreeDebit,
				       const INT16       DebitCodeNumber,
				       const INT16       ReadCodeNumber,
				       const INT16       AccessCondLevel,
				       const INT16       CreditCodeNumber,
					     BYTE  G_FAR *MACIn,
					     BYTE  G_FAR *MACOut
				     );
INT16 G_DECL G_MPC_LoadKeyPCOS      ( const WORD16       Reader,
				      const INT16        KeyFileNum,
				      const INT16        KeyNumber,
				      const INT16        TypeKey,  
				      const INT16        KeyVersion,
					    BYTE   G_FAR *KeyValue
				    );  
INT16 G_DECL G_MPC_LoadKey          ( const WORD16       Reader,
				      const INT16        Mode,
				      const INT16        KeyFileNum,
				      const INT16        KeyNumber,
				      const INT16        TypeKey,   
				      const INT16        KeyVersion,
					    BYTE   G_FAR *KeyValue
				    );
													     
INT16 G_DECL G_MPC_LoadKeySecure    ( const WORD16       Reader,
				      const INT16        SMType,
				      const INT16        Mode,
				      const INT16        KeyFileNum,
				      const INT16        KeyNumber,
				      const INT16        TypeKey,
				      const INT16                KeyVersion,
					    BYTE   G_FAR *KeyValue,
					    BYTE   G_FAR *MACOut
				    ); 
													   
INT16 G_DECL G_MPC_InitBalance      ( const WORD16       Reader,
				      const INT16        PurseFile,
				      const INT32        BalanceAmount
				    );
void G_DECL G_MPC_LoadACC           ( ACC    G_FAR *Grp,
				      INT16        Validity,
				      INT16        KeyLevel,
				      INT16        KeyIdentifier,
				      INT16        Sc1Level,
				      INT16        Sc1Number,
				      INT16        Sc2Level,
				      INT16        Sc2Number
				    );
INT16  G_DECL G_MPC_VerifyACC       (       ACC    G_FAR *ContextGrp );

INT16  G_DECL G_MPC_VerifyACF       (       ACF    G_FAR *ContextGrp );

void G_DECL G_MPC_ComputeMSC        (       BYTE   G_FAR *Code,
					    BYTE   G_FAR *Alea,
					    BYTE   G_FAR *CardRefNumber
				    );

INT16 G_DECL G_MPC_Erase(const INT16 Reader);

INT16 G_DECL G_MPC_SetLock(const INT16 Reader);

/******************************************************************************/
/* Administrative commands prototypes                                         */
/******************************************************************************/

INT16 G_DECL G_MPC_VerifyMSC        ( const WORD16       Reader, 
					    WORD8  G_FAR *MSC_Value
				    );
INT16 G_DECL G_MPC_VerifyCode       ( const WORD16       Reader,
				      const INT16        CodeNumber,
					    WORD8  G_FAR *CodeValue
				    );
INT16 G_DECL G_MPC_VerifyCodeSecure ( const WORD16       Reader,
				      const INT16        SMType,
				      const INT16        CodeNumber,
					    WORD8  G_FAR *CodeValue,
					    WORD8  G_FAR *CodeCi
				    );
INT16 G_DECL G_MPC_SetCode          ( const WORD16       Reader,
				      const INT16        Mode,
				      const INT16        CodeNumber,
					    WORD8  G_FAR *CheckCodeValue,
					    WORD8  G_FAR *NewCodeValue
				    );
INT16 G_DECL G_MPC_SetCodeSecure    ( const WORD16       Reader,
				      const INT16        SMType,
				      const INT16        Mode,
				      const INT16        CodeNumber,
					    WORD8  G_FAR *CheckCodeValue,
					    WORD8  G_FAR *NewCodeValue,
					    BYTE   G_FAR *CodeCi
				    );
INT16 G_DECL G_MPC_CreateDF         ( const WORD16       Reader,
				      const WORD16       Identifier,
				      const INT16        GeneralOpt,
				      const INT16        OnLineOpt,
				      const INT16        CryptoOpt,
				      const INT16        SizeDFName,
					    ACC    G_FAR *Groupe1,
					    ACC    G_FAR *Groupe2,
					    ACC    G_FAR *Groupe3,
					    BYTE   G_FAR DFName[]
				    );
INT16 G_DECL G_MPC_CreateDFSecure   ( const WORD16       Reader,
				      const INT16        SMType,
				      const WORD16       Identifier,
				      const INT16        GeneralOpt,
				      const INT16        OnLineOpt,
				      const INT16        CryptoOpt,
				      const INT16        SizeDFName,
					    ACC    G_FAR *Group1,
					    ACC    G_FAR *Group2,
					    ACC    G_FAR *Group3,
					    BYTE   G_FAR DFName[],
					    BYTE   G_FAR *MACIn,
					    BYTE   G_FAR *MACOut
				    );
INT16 G_DECL G_MPC_CreateEF         ( const WORD16       Reader,
				      const WORD16       Identifier,
				      const INT16        TypeEF,
				      const INT16        SizeEF,
					    ACC    G_FAR *Group1,
					    ACC    G_FAR *Group2,
					    ACC    G_FAR *Group3
				    );
INT16 G_DECL G_MPC_CreateEFSecure   ( const WORD16       Reader,
				      const INT16        SMType,
				      const WORD16       Identifier,
				      const INT16        TypeEF,
				      const INT16        SizeEF,
					    ACC    G_FAR *Group1,
					    ACC    G_FAR *Group2,
					    ACC    G_FAR *Group3,
					    BYTE   G_FAR *MACIn,
					    BYTE   G_FAR *MACOut
				    );
INT16 G_DECL G_MPC_Select           ( const WORD16       Reader,
				      const INT16        Mode,
				      const WORD16       FileId,
					    BYTE   G_FAR *AskGetResp
				    );
INT16 G_DECL G_MPC_SelectDFName     ( const WORD16       Reader,
				      const INT16        LengthIn,
					    BYTE   G_FAR *DFname,
					    BYTE   G_FAR *AskGetResp
				    );
INT16 G_DECL G_MPC_SelectSimple     ( const WORD16       Reader,
				      const INT16        Mode,
				      const WORD16       FileId
				    );
INT16 G_DECL G_MPC_SelectDFNameSimple(const WORD16       Reader,
				      const INT16        LengthIn,
					    BYTE   G_FAR *DFname
				    );
INT16 G_DECL G_MPC_GetInfo          ( const WORD16       Reader,
				      const INT16        Mode,
				      const WORD16       Length,
					    WORD8  G_FAR *Response
				    );
INT16 G_DECL G_MPC_Freeze           ( const WORD16       Reader,
				      const INT16        Mode,
				      const INT16        Identifier,
					    ACF    G_FAR *Groupe1,
					    ACF    G_FAR *Groupe2,
					    ACF    G_FAR *Groupe3
				    );
INT16 G_DECL G_MPC_FreezeSecure     ( const WORD16       Reader,
				      const INT16        SMType,
				      const INT16        Mode,
				      const INT16        Identifier,
					    ACF    G_FAR *Groupe1,
					    ACF    G_FAR *Groupe2,
					    ACF    G_FAR *Groupe3,
					    BYTE   G_FAR *MACIn,
					    BYTE   G_FAR *MACOut
				    );
INT16 G_DECL G_MPC_SwitchProtocol (const WORD16  Reader,
				   const INT32  Speed,
				   const INT16  Protocol,
				   const INT16  GuardTime
				  );

INT16 G_DECL G_MPC_ActivateSysBasic ( const WORD16       Reader,
				      const INT16        SMType,
				      const INT16        NumSysMechanism,
				      const INT16        Address,
					    BYTE   G_FAR *MACIn,
					    BYTE   G_FAR *MACOut
				    );
INT16 G_DECL G_MPC_ActivateSysATR   ( const WORD16       Reader,
				      const INT16        SMType,
				      const INT16        NumSysMechanism,
				      const INT16        LengthATR,
				      const INT16        AddressATR,
				      const INT32        Speed,
				      const INT16        Protocol,
				      const INT16        ExtraGuardTime,
					    BYTE   G_FAR *MACIn,
					    BYTE   G_FAR *MACOut
				    );
INT16 G_DECL G_MPC_SelKey           ( const WORD16       Reader,
				      const INT16        KeyLevel,
				      const WORD16       KeyFileId,
				      const INT16        KeyNumber,
					    WORD8  G_FAR *TerminalRandom,
					    WORD8  G_FAR *Result,
					    WORD8  G_FAR *CardRandomNumber
				    );
INT16 G_DECL G_MPC_SelKeySimple     ( const INT16        Reader,
				      const INT16        KeyLevel,
				      const WORD16       KeyFileName,
				      const INT16        KeyNumber,
					    WORD8  G_FAR *TerminalRandom
				    );
INT16 G_DECL G_MPCSM_SetSecureSessionKey
				    ( const INT16        SMType,
				      const INT16        KeyLevel,    
				      const WORD16       KeyFileId,
				      const INT16        KeyNumber,
					    WORD8  G_FAR *TerminalRandom,
					    WORD8  G_FAR *CardRandom,
					    WORD8  G_FAR *Result,
					    WORD8  G_FAR *KeyRef
				    );

/******************************************************************************/
/* R/W/U commands prototype                                                   */
/******************************************************************************/
WORD16 G_DECL G_MPC_GetIOSize       ( void                  );
void   G_DECL G_MPC_SetIOSize           ( const WORD16 size     ); 

INT16 G_DECL G_MPC_ReadBin          ( const WORD16       Reader,
				      const INT16        Mode,
				      const WORD16       Identifier,
				      const WORD16       Offset,
				      const WORD16       Size,
					    BYTE   G_FAR *DataRead
				    );
INT16 G_DECL G_MPC_ReadBinSecure    ( const WORD16       Reader,
				      const INT16        SMType,
				      const INT16        Mode,
				      const WORD16       Identifier,
				      const WORD16       Offset,
				      const WORD16       Size,
					    BYTE   G_FAR *DataRead,
					    BYTE   G_FAR *MACOut
				    );
INT16  G_DECL G_MPC_EditReadBin     ( const WORD16       Reader,
				      const INT16        Mode,
				      const WORD16       Identifier,
				      const WORD16       Offset,
				      const WORD16       Size,
					    BYTE   G_FAR *DataRead
				    );
INT16 G_DECL G_MPC_EditReadBinSecure (const WORD16 Reader,
				      const INT16  SMType,
				      const INT16  Mode,
				      const WORD16 Identifier,
				      const WORD16 Offset,
				      const WORD16 Size,
					    BYTE  G_FAR *DataRead,
					    BYTE  G_FAR *MACOut
				     );

INT16 G_DECL G_MPC_WriteBin         ( const WORD16       Reader,
				      const INT16        Mode,
				      const WORD16       Identifier,
				      const WORD16       Offset,
				      const WORD16       Size,
					    BYTE   G_FAR *DataWrite
				    );
INT16 G_DECL G_MPC_WriteBinSecure   ( const WORD16       Reader,
				      const INT16        SMType,
				      const INT16        Mode,
				      const WORD16       Identifier,
				      const WORD16       Offset,
				      const WORD16       Size,
					    BYTE   G_FAR *DataWrite,
					    BYTE   G_FAR *MACIn,
					    BYTE   G_FAR *MACOut
				    );
INT16 G_DECL G_MPC_EditWriteBin                 ( const WORD16  Reader,
					  const INT16  Mode,
					  const WORD16 Identifier,
				     const WORD16 Offset,
					  const WORD16 Size,
						  BYTE  G_FAR *DataWrite
						);
INT16 G_DECL G_MPC_EditWriteBinSecure (const WORD16 Reader,
				       const INT16  SMType,
				       const INT16  Mode,
				       const WORD16 Identifier,
				       const WORD16 Offset,
				       const WORD16 Size,
					     BYTE  G_FAR *DataWrite,
					     BYTE  G_FAR *MACIn,
					     BYTE  G_FAR *MACOut
				      );


INT16 G_DECL G_MPC_UpdateBin        ( const WORD16       Reader,
				      const INT16        Mode,
				      const WORD16       Identifier,
				      const WORD16       Offset,
				      const WORD16       Size,
					    BYTE   G_FAR *DataWrite
				    );
INT16 G_DECL G_MPC_UpdateBinSecure  ( const WORD16       Reader,
				      const INT16        SMType,
				      const INT16        Mode,
				      const WORD16       Identifier,
				      const WORD16       Offset,
				      const WORD16       Size,
					    BYTE   G_FAR *DataWrite,
					    BYTE   G_FAR *MACIn,
					    BYTE   G_FAR *MACOut
				    );
INT16 G_DECL G_MPC_EditUpdateBin        ( const WORD16 Reader,
					  const INT16  Mode,
					  const WORD16 Identifier,
					  const WORD16 Offset,
					  const WORD16 Size,
						  BYTE  G_FAR *DataWrite
					);
INT16 G_DECL G_MPC_EditUpdateBinSecure (const WORD16 Reader,
					const INT16  SMType,
					const INT16  Mode,
					const WORD16 Identifier,
					const WORD16 Offset,
					const WORD16 Size,
					      BYTE  G_FAR *DataWrite,
					      BYTE  G_FAR *MACIn,
					      BYTE  G_FAR *MACOut
				       );


INT16 G_DECL G_MPC_ReadFile         ( const WORD16       Reader,
				      const INT16        FileNumber,
				      const INT16        FileOffset,
				      const WORD16       Size,
					    BYTE  G_FAR *Data_Read
				    );
INT16 G_DECL G_MPC_EditReadFile                 ( const WORD16 Reader,
					  const INT16 FileNumber,
					  const INT16 FileOffset,
					  const INT16 Size,
					  WORD8 far *Data_Read
						);

INT16 G_DECL G_MPC_UpdateFile       ( const WORD16       Reader,
				      const INT16        FileNumber,
				      const INT16        FileOffset,
				      const WORD16       Size,
					    WORD8  G_FAR *Data_Write
				    );
INT16 G_DECL G_MPC_EditUpdateFile       ( const WORD16 Reader,
					  const INT16 FileNumber,
					  const INT16 FileOffset,
					  const INT16 Size,
						  WORD8 far *Data_Write
					);


/******************************************************************************/
/* Payment commands prototype                                                 */
/******************************************************************************/

INT16 G_DECL G_MPC_SelPurseKey      ( const WORD16       Reader,
				      const WORD16       KeyFile,
				      const INT16        KeyNumber,
				      const WORD16       PurseFile,
					    WORD8  G_FAR *Random,
					    WORD8  G_FAR *Result,
					    INT32  G_FAR *CardTransactionCounter
				    );
INT16 G_DECL G_MPC_SelPurseKeySimple( const INT16        Reader,
				      const WORD16       Key_File,
				      const INT16        Key_Number,
				      const WORD16       Purse_File,
					    WORD8  G_FAR *Random
				    );
INT16 G_DECL G_MPCSM_SetPaymentSessionKey(const INT16    SMType,
				      const WORD16       PurseFile,
				      const INT32        CardTransactionCounter,
					    WORD8  G_FAR *Random,
					    WORD8  G_FAR *CardResult,
					    WORD8  G_FAR *PurseKeyRef
				    );
INT16 G_DECL G_MPC_DebitPurse       ( const WORD16       Reader,
				      const INT32        DebitAmount,
				      const WORD16       TerminalIdentifier,
				      const WORD16       TerminalCounter,
					    WORD8  G_FAR *CardDebitCertificate
				    );
INT16 G_DECL G_MPC_DebitPurseSimple ( const WORD16       Reader,
				      const INT32        DebitAmount,
				      const WORD16       TerminalIdentifier,
				      const WORD16       TerminalCounter
				    );
INT16 G_DECL G_MPCSM_SubstituteDebitCryptogram
				    ( const INT16        SMType,
				      const INT32        DebitAmount,
				      const WORD16       TerminalIdentifier,
				      const WORD16       Terminalcounter,
					    WORD8  G_FAR *Cryptogram
				    );

INT16 G_DECL G_MPC_SubstituteDebitSimple
				    ( const WORD16       Reader,
				      const INT32        LastDebitAmount,
					    WORD8  G_FAR *Cryptogram
				    );
INT16 G_DECL G_MPC_SubstituteDebit  ( const WORD16       Reader,
				      const INT32        Last_Debit_Amount,
					    WORD8  G_FAR *Sdbt_Cryptogram,
					    WORD8  G_FAR *Card_Dbt_Certificate
				    );
INT16 G_DECL G_MPCSM_CheckDebitCertificate
				    ( const INT16        SMType,
				      const WORD16       TerminalIdentifier,
				      const WORD16       TerminalCounter,
				      const INT32        DebitAmount,
					    WORD8  G_FAR *Card_Dbt_Certificate
				    );
INT16 G_DECL G_MPCSM_HostCreditCryptogram
				    ( const INT16        SMType,
				      const INT32        CreditAmount,
					    WORD8  G_FAR *CreditKey,
					    WORD8  G_FAR *HostCreditCryptogram
				    );
INT16 G_DECL G_MPCSM_TermCreditCryptogram
				    ( const INT16        SMType,
					    WORD8  G_FAR *HostCreditCryptogram,
					    WORD8  G_FAR *TermCreditCryptogram
				    );

INT16 G_DECL G_MPC_CreditPurse      ( const WORD16       Reader,
				      const INT16        KeyNumber,
				      const INT32        CreditAmount,
					    WORD8  G_FAR *CreditCryptogram
				    );
INT16 G_DECL G_MPC_Sign             ( const WORD16       Reader,
				      const WORD16       SignKeyFile,
				      const INT16        SignKeyNumber,
					    WORD8  G_FAR *CardSignCertificate
				    );
INT16 G_DECL G_MPCSM_CheckSignCertificate
				    ( const INT16        SMType,
				      const INT32        Amount,
				      const INT32        Balance,
					    WORD8  G_FAR *SignKeyRef,
					    WORD8  G_FAR *CardSignCertificate
				    );
INT16 G_DECL G_MPC_ReadBalance      ( const WORD16       Reader,
				      const WORD16       PurseFile,
				      const WORD16       TerminalIdentifier,
				      const WORD16       TerminalCounter,
					    INT32  G_FAR *Balance,
					    WORD8  G_FAR *CardBalanceCertificate
				    );
INT16 G_DECL G_MPCSM_CheckBalanceCertificate
				    ( const INT16        SMType,
				      const WORD16       PurseFile,
				      const WORD16       TerminalIdentifier,
				      const WORD16       TerminalCounter,
				      const INT32        Balance,
					    WORD8  G_FAR *CardBalanceCertificate
				    );
INT16 G_DECL G_MPC_SetOption        ( const WORD16       Reader,
				      const INT16        Option
				    );
INT16 G_DECL G_MPC_SignOnLine       ( const WORD16       Reader,
				      const WORD16       SignKeyLevel,
				      const WORD16       SignKeyFile,
				      const INT16        SignKeyNumber,
					    INT32  G_FAR *SCC,
					    WORD8  G_FAR *Header,
					    WORD8  G_FAR *CardSignCertificate
				    );
INT16 G_DECL G_MPCSM_CheckSignOnLineCertificate
				    ( const INT16        SMType,
				      const INT32        SCC,
					    WORD8  G_FAR *Header,
					    WORD8  G_FAR *SignKeyRef,
					    WORD8  G_FAR *CardSignCertificate
				    );


/******************************************************************************/
/* Security module commands prototype                                         */
/******************************************************************************/
INT16 G_DECL G_SM_GetResponse (const WORD16 SMReader,
			       const WORD16 Length,
				     WORD8 G_FAR *Response
			       );


INT16 G_DECL G_SM_ChoKey            (       INT16        Reader,
					    BYTE         p_lo,
					    BYTE         p_no_file,
					    BYTE         p_lng,
					    BYTE  G_FAR *p_data
				    );
INT16 G_DECL G_SM_GenKey            (       INT16        Reader, 
					    INT16        flag1, 
					    INT16        flag2, 
					    BYTE  G_FAR *result
				    );
INT16 G_DECL G_SM_CalMac            (       INT16        Reader, 
					    BYTE  G_FAR *p_data
				    );
INT16 G_DECL G_SM_CalEnc            ( const INT16        Reader,
					    BYTE  G_FAR *p_data,
					    BYTE  G_FAR *p_res
				    );
INT16 G_DECL G_SM_CalDec            ( const INT16        Reader, 
					    BYTE G_FAR *p_data,
					    BYTE G_FAR *p_res
				    );
		     
INT16 G_DECL G_SM_ReadMem (const WORD16 Reader, 
			   const WORD16 Address, 
			   const WORD16 Size, 
				 WORD8 G_FAR *DataRead
			  );
			  
INT16 G_DECL G_SM_ReadFile (const WORD16 Reader, 
			    const INT16  FileNumber,
			    const INT16  FileOffset,
			    const WORD16 Size, 
				 WORD8 G_FAR *DataRead
			  );

INT16 G_DECL G_SM_ComputeMSC (BYTE G_FAR *CSN_B,
			      BYTE G_FAR *BSN,
			      BYTE G_FAR *MSC_B
			     );

INT16 G_DECL G_SM_ComputeKeySyst (BYTE G_FAR *CSN_MPC,
				  BYTE G_FAR *MKeySyst,
				  BYTE G_FAR *KeySyst
				 );
				 
INT16 G_DECL G_SM_VerifyMSC (const WORD16 Reader, WORD8 far *MSC_Value);         

INT16 G_DECL G_InitBatch ( const WORD16 BatchKey,const WORD16 SMReader );


#endif
